@@ -6,7 +6,10 @@ __all__ = [
     "AferoState",
     "EventType",
     "InvalidAuth",
+    "InvalidOTP",
     "InvalidResponse",
+    "OTPError",
+    "OTPRequired",
     "SecuritySystemError",
     "anonymize_device",
     "anonymize_devices",
@@ -30,5 +33,13 @@ finally:
 from . import v1
 from .anonomyize_data import anonymize_device, anonymize_devices
 from .device import AferoDevice, AferoState, get_afero_device
-from .errors import AferoError, InvalidAuth, InvalidResponse, SecuritySystemError
+from .errors import (
+    AferoError,
+    InvalidAuth,
+    InvalidOTP,
+    InvalidResponse,
+    OTPError,
+    OTPRequired,
+    SecuritySystemError,
+)
 from .types import EventType
