@@ -104,6 +104,7 @@ class AferoBridgeV1:
         afero_client: str | None = "hubspace",
         hide_secrets: bool = True,
         poll_version: bool = True,
+        client_name: str | None = "aioafero",
     ):
         """Create a bridge that communicates with Afero IoT API."""
         if hide_secrets:
@@ -122,6 +123,7 @@ class AferoBridgeV1:
             afero_client=afero_client,
             hide_secrets=hide_secrets,
         )
+        self.client_name = client_name
         self.logger = logging.getLogger(f"{__package__}-{afero_client}[{username}]")
         if len(self.logger.handlers) == 0:
             self.logger.addHandler(logging.StreamHandler())
@@ -171,6 +173,13 @@ class AferoBridgeV1:
         """Get all tracked devices."""
         return set(self._known_devs.keys())
 
+    async def otp_login(self, otp_code: str) -> None:
+        """Perform OTP login with the provided code."""
+        task = asyncio.create_task(self._auth.perform_otp_login(otp_code))
+        self.add_job(task)
+        await task
+        return task.result()
+
     def add_device(
         self, device_id: str, controller: BaseResourcesController[AferoResource]
     ) -> None:
@@ -291,20 +300,28 @@ class AferoBridgeV1:
         return self._account_id
 
     async def initialize(self) -> None:
-        """Initialize the bridge for communication with Afero API."""
+        """Initialize the bridge for communication with Afero API.
+
+        To ensure the bridge is fully initialized, call async_block_until_done().
+        """
         if len(self._scheduled_tasks) == 0:
-            await self.initialize_cleanup()
             await self.get_account_id()
-            await asyncio.gather(
-                *[
-                    controller.initialize()
-                    for controller in self._controllers.values()
-                    if not controller.initialized
-                ]
-            )
-            await self._events.initialize()
+            for controller in self._controllers.values():
+                if controller.initialized:
+                    continue
+                self.add_job(asyncio.create_task(controller.initialize()))
+            self.add_job(asyncio.create_task(self.initialize_cleanup()))
+            self.add_job(asyncio.create_task(self.events.initialize()))
+            self.add_job(asyncio.create_task(self.events.wait_for_first_poll()))
 
     async def fetch_data(self, version_poll=False) -> list[dict[Any, str]]:
+        """Query the API."""
+        task = asyncio.create_task(self._fetch_data(version_poll))
+        self.add_job(task)
+        await task
+        return task.result()
+
+    async def _fetch_data(self, version_poll=False) -> list[dict[Any, str]]:
         """Query the API."""
         self.logger.debug("Querying API for all data")
         headers = {
@@ -339,6 +356,13 @@ class AferoBridgeV1:
         return data
 
     async def fetch_device_states(self, device_id) -> list[dict[Any, str]]:
+        """Query the API for new device states."""
+        task = asyncio.create_task(self._fetch_device_states(device_id))
+        self.add_job(task)
+        await task
+        return task.result()
+
+    async def _fetch_device_states(self, device_id) -> list[dict[Any, str]]:
         """Query the API for new device states."""
         self.logger.debug("Querying the API for updated states for %s", device_id)
         headers = {
@@ -445,7 +469,9 @@ class AferoBridgeV1:
     def get_headers(self, **kwargs):
         """Get default headers for an API call."""
         headers: dict[str, str] = {
-            "user-agent": v1_const.AFERO_GENERICS["DEFAULT_USERAGENT"],
+            "user-agent": v1_const.AFERO_GENERICS["DEFAULT_USERAGENT"].safe_substitute(
+                client_name=self.client_name
+            ),
             "accept-encoding": "gzip",
         }
         headers.update(kwargs)
