@@ -117,10 +117,6 @@ class LightController(BaseResourcesController[Light]):
         "dimming": "brightness",
         "effect": "color-sequence",
     }
-    # Sensors map functionClass -> Unit
-    ITEM_SENSORS: dict[str, str] = {}
-    # Binary sensors map key -> alerting value
-    ITEM_BINARY_SENSORS: dict[str, str] = {}
     # Split Lights from the primary payload
     DEVICE_SPLIT_CALLBACKS: dict[str, callable] = {
         ResourceTypes.LIGHT.value: light_callback
@@ -213,11 +209,6 @@ class LightController(BaseResourcesController[Light]):
                 color_mode = features.ColorModeFeature(state.value)
             elif state.functionClass == "available":
                 available = state.value
-            elif sensor := await self.initialize_sensor(state, afero_device.device_id):
-                if isinstance(sensor, AferoBinarySensor):
-                    binary_sensors[sensor.id] = sensor
-                else:
-                    sensors[sensor.id] = sensor
 
         supported_color_modes: list[str] = []
         for function in afero_device.functions:
@@ -310,8 +301,6 @@ class LightController(BaseResourcesController[Light]):
                 if cur_item.available != state.value:
                     cur_item.available = state.value
                     updated_keys.add("available")
-            elif update_key := await self.update_sensor(state, cur_item):
-                updated_keys.add(update_key)
 
         # Several states hold the effect, but its always derived from the preset functionInstance
         return updated_keys.union(
