@@ -495,6 +495,13 @@ async def ex1_rc(bridge_with_acct_req):
     yield bridge_with_acct_req.ex1
 
 
+@pytest_asyncio.fixture
+async def ex1_rc_mocked(mocked_bridge):
+    mocked_bridge.add_controller("ex1", Example1ResourceController)
+    await mocked_bridge.initialize()
+    yield mocked_bridge.ex1
+
+
 @pytest_asyncio.fixture
 async def ex2_rc(bridge_with_acct_req):
     bridge_with_acct_req.add_controller("ex2", Example2ResourceController)
@@ -502,6 +509,13 @@ async def ex2_rc(bridge_with_acct_req):
     yield bridge_with_acct_req.ex2
 
 
+@pytest_asyncio.fixture
+async def ex2_rc_mocked(mocked_bridge):
+    mocked_bridge.add_controller("ex2", Example2ResourceController)
+    await mocked_bridge.initialize()
+    yield mocked_bridge.ex2
+
+
 def test_init(ex1_rc):
     assert isinstance(ex1_rc._bridge, AferoBridgeV1)
     assert ex1_rc._items == {}
@@ -1093,8 +1107,9 @@ async def test_update_dev_not_found(ex1_rc, caplog):
     ],
 )
 async def test_update(
-    obj_in, states, expected_states, expected_item, successful, ex1_rc, mocker
+    obj_in, states, expected_states, expected_item, successful, ex1_rc_mocked, mocker
 ):
+    ex1_rc = ex1_rc_mocked
     await ex1_rc.initialize()
     mocker.patch("time.time", return_value=12345)
     await ex1_rc._bridge.events.generate_events_from_data(
@@ -1169,8 +1184,9 @@ async def test_update(
     ],
 )
 async def test_update_dict(
-    device, obj_in, states, expected_states, expected_item, successful, ex2_rc, mocker
+    device, obj_in, states, expected_states, expected_item, successful, ex2_rc_mocked, mocker
 ):
+    ex2_rc = ex2_rc_mocked
     await ex2_rc.initialize()
     mocker.patch("time.time", return_value=12345)
     ex2_rc._items[device.id] = await ex2_rc.initialize_elem(device)
@@ -1554,3 +1570,20 @@ def test_get_afero_state_from_feature(
 def test_get_afero_states_from_list(states, expected, mocker):
     mocker.patch("time.time", return_value=12345)
     assert get_afero_states_from_list(states) == expected
+
+
+def test_unsubscribe(ex1_rc):
+    assert ex1_rc._subscribers == {"*": []}
+
+    def whatever(*args, **kwargs):
+        pass
+
+    unsub = ex1_rc.subscribe(whatever, id_filter="beans")
+    unsub2 = ex1_rc.subscribe(whatever, id_filter="beans2")
+    assert ex1_rc._subscribers == {"*": [], "beans": [(whatever, None)], "beans2": [(whatever, None)]}
+    ex1_rc._subscribers = {"*": [], "beans": [(whatever, None)]}
+    # Ensure no error if unsub called if its somehow removed
+    unsub2()
+    ex1_rc._subscribers = {"*": [], "beans": [(whatever, None)]}
+    unsub()
+    assert ex1_rc._subscribers == {"*": [], "beans": []}
\ No newline at end of file
